import asyncio
from io import StringIO

# noinspection PyPackageRequirements
import discord
from v6d0auth.run_app import start_app
from v6d1tokens.client import request_token

from v6d3losyash import config
from v6d3losyash.app import get_app

loop = asyncio.get_event_loop()
token = loop.run_until_complete(request_token('losyash'))
client = discord.Client(
    intents=discord.Intents(
        members=True,
        guilds=True,
        emojis=True,
        reactions=True,
        invites=True,
        bans=True,
    ),
)
ESCAPED = '`_*\'"\\'


def escape(s: str):
    res = StringIO()
    for c in s:
        if c in ESCAPED:
            c = '\\' + c
        res.write(c)
    return res.getvalue()


@client.event
async def on_ready():
    print("ready")
    await client.change_presence(activity=discord.Game(
        name='феноменально',
    ))


@client.event
async def on_raw_reaction_add(payload: discord.RawReactionActionEvent):
    emoji: discord.Emoji = payload.emoji
    if emoji.id != config.emoji:
        return
    if payload.message_id != config.message:
        return
    guild: discord.Guild = client.get_guild(config.guild)
    member: discord.Member = guild.get_member(payload.user_id)
    role = guild.get_role(config.role)
    if role in member.roles:
        return
    await member.add_roles(role, reason='феноменально')

    channel: discord.TextChannel = guild.get_channel(config.channel)
    await channel.send(f'{escape(str(member))} <:Jesus:586669134406877270>')


@client.event
async def on_member_join(member: discord.Member):
    guild: discord.Guild = member.guild
    if guild.id != config.guild:
        return
    channel: discord.TextChannel = guild.get_channel(config.channel)
    await channel.send(f'{escape(str(member))} joined')


@client.event
async def on_member_remove(member: discord.Member):
    guild: discord.Guild = member.guild
    if guild.id != config.guild:
        return
    channel: discord.TextChannel = guild.get_channel(config.channel)
    await channel.send(f'{escape(str(member))} left')


async def main():
    await start_app(get_app(client))
    await client.login(token)
    await client.connect()


if __name__ == '__main__':
    loop.run_until_complete(main())
